<?php
namespace Modules\Cotacao\Modules\Pedido\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Util\SupUtilities;

class ControlePedidoColetaAceite extends \Teknisa\GER\Controllers\TeknisaCrud
{

    protected $crudService;
    protected $controlePedidoColetaAceiteService;
    protected $controlePedidoColetaService;
    protected $dataSourceManager;
    protected $smtpUtil;
    protected $entityManager;
    protected $zhFileUploader;
    protected $fileServerURL;
    protected $supUtilities;

    public function __construct($crudService, $parameterBag, $controlePedidoColetaAceiteService, $controlePedidoColetaService, $dataSourceManager, $smtpUtil, EntityManager $entityManager, $zhFileUploader, $fileServerURL, SupUtilities $supUtilities) 
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->controlePedidoColetaAceiteService = $controlePedidoColetaAceiteService;
        $this->controlePedidoColetaService = $controlePedidoColetaService;
        $this->dataSourceManager = $dataSourceManager;
        $this->smtpUtil = $smtpUtil;
        $this->entityManager = $entityManager;
        $this->templatesDir = realpath(__DIR__ . '/../../../../../Util/Template');
        $this->zhFileUploader    = $zhFileUploader;
        $this->fileServerURL     = $fileServerURL;
        $this->supUtilities = $supUtilities;
    }
    
    public function verifyParamTermoAceite($cdFilial, $pedidos, $nmOperadorLogado, $emailLogado, $pedidosPai, $response, $rotina = false){
        $count = 0;

        foreach ($pedidosPai as $pedidoPai){
            $paramAceite = $this->controlePedidoColetaAceiteService->returnParamTermoAceite($pedidoPai['CDFORNECED']);
            if($paramAceite === 'S'){
                foreach ($pedidos as $pedido){
                    if($pedido['NRPEDIDOCOL'] == $pedidoPai['ID']){
                        $this->controlePedidoColetaService->alteraStatus($pedido['NRPEDIDOCOL'], $pedido['CDFILIAL'], $pedido['CDFORNECED'], $pedido['NRPEDCOLETA']);
                    }
                }
                $this->enviaEmailAguardandoAceite($cdFilial, [$pedidoPai['CDFORNECED']], $pedidoPai['NRPEDIDOCOL'], $nmOperadorLogado, $emailLogado, $response);
                $count++;
            }
        }
        
        if($count){
            $this->controlePedidoColetaService->commit();
            throw new \Exception('AGUARD_ACEITE_FORN', 13);
        }
    }
    
    public function enviaEmailAguardandoAceite($cdFilial, $cdForneced, $nrPedido, $nmOperadorLogado, $emailLogado, $response){
        try{
            $assunto = 'Liberação do aceite do pedido de cotação';
            $message = $this->getMessageAguardandoAceite($nrPedido, $nmOperadorLogado);
            $messageSendMail = $this->supUtilities->messageSendMail($cdFilial, $assunto, $message, 'PORTAL E-COMMERCE');
            $emailFornecedor = $this->supUtilities->retornarEmailsRelacionadosFornecedor($cdForneced);
            
            $this->supUtilities->enviaEmail($emailLogado, $assunto, $emailFornecedor, $messageSendMail, $emailFornecedor);
        }catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(),500));
        }
    }
    
	public function authOperConfirmaRecusaTermo(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        try {
            $this->entityManager->beginTransaction();
			$operadorAutorizado = $this->controlePedidoColetaAceiteService->validatePassword($row['OPERADORLOGADO'], $row['SENHA']);
			
			if ($operadorAutorizado) {
			    $return = $this->defineOperador($row, $response);
			}

			if ($return["ERROR"]) {
			    $response->setError(new \Zeedhi\Framework\DTO\Response\Error($return, 1));
			} else {
			    $this->entityManager->commit();
			}
		} catch (\Exception $e) {
		    $this->entityManager->rollback();
		    $erroMessage = $e->getMessage();
		    
		    if ($e->getCode() === 1) {
		        $erroMessage = array("ERROR" => true, "CODE" => $erroMessage, "PARAM" => "");
		    }
		    
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($erroMessage, 1));
		}
    }
    
    public function defineOperador($row, $response) 
    {   
        if ($row['OPERADORLOGADO'] === $row['PEDIDO'][0]['CDOPERLANC']) {
            $return = $this->isOperLanc($row, $response);
        } else {
            $return = $this->noIsOperLanc($row, $response);
        }

        return is_array($return) ? $return : array("ERROR" => false);
    }
     
    public function isOperLanc($row, $response){

        $dataset = array();
        foreach ($row['PEDIDO'] as $pedido){
            
            $idAceiteOper = $this->controlePedidoColetaAceiteService->returnIdAceite($pedido);

            if ($idAceiteOper) {
                $nextOper = $this->controlePedidoColetaAceiteService->returnNextOperGereComp($pedido);

                if ($idAceiteOper === 'N') {
                    if ($row['ID_AGREED'] === 'S') {
                        $this->controlePedidoColetaAceiteService->updateAceitePedCol($pedido, $row['OPERADORLOGADO']);
                        if (!empty($nextOper)) {
                            $this->controlePedidoColetaAceiteService->insertNextOperAceitePedCol($pedido, $nextOper[0]['CDOPERGERE']);
                            $this->enviaEmailNextOper($row, $nextOper,$pedido);
                             array_push($dataset, array(
                                "GERACOTACAO" => false,
                                "RECUSA"      => false
                            ));
                            
                            //$response->addDataSet(new DataSet('data', $dataset));
                        } else {
                            $this->finalizeProcessAccepted($pedido, $response);
                            array_push($dataset, array(
                                "GERACOTACAO" => true
                               
                            ));
                        }
                    } else {
                        $this->controlePedidoColetaAceiteService->updateAceitePedColRecusa($pedido, $row['OPERADORLOGADO'], $row['JUSTIFICATIVA']);
                        $this->controlePedidoColetaAceiteService->updateStatusPedCol($pedido, 'N');
                        $this->enviaEmailRecusa($row,$pedido);
                        array_push($dataset, array(
                            "GERACOTACAO" => false,
                            "RECUSA"      => true
                        ));
                        //$response->addDataSet(new DataSet('data', $dataset));
                    }
                } else {
                    $nextOperPending = $this->controlePedidoColetaAceiteService->returnNextOperAceitePedCol($pedido);
                   
                    if (!empty($nextOperPending)) {
                        array_push($dataset, array(
                            "GERACOTACAO" => false,
                            "RECUSA"      => false
                        ));
                        //$response->addDataSet(new DataSet('data', $dataset));
                        return array("ERROR" => true, "CODE" => "OPERADOR_APROVADOR_DIF", "PARAM" => $nextOperPending[0]['NMOPERADOR']);
                        
                    }
                }
            } else {
                return array("ERROR" => true, "CODE" => "AGUARD_APROV_FORN", "PARAM" => "");
               
            }
        }
        
        $response->addDataSet(new DataSet('data', $dataset));

        
    }
    
    public function noIsOperLanc($row, $response)
    {
        $dataset = array();
        foreach ($row['PEDIDO'] as $pedido){
            $nextOperPending = $this->controlePedidoColetaAceiteService->returnNextOperAceitePedCol($pedido);
            
            if ($nextOperPending) {
                
                if ($row['OPERADORLOGADO'] == $nextOperPending[0]['CDOPERADOR']) {
                    
                    if ($row['ID_AGREED'] === 'S') {
                        
                        $this->controlePedidoColetaAceiteService->updateAceitePedCol($pedido, $row['OPERADORLOGADO']);
                        $nextOper = $this->controlePedidoColetaAceiteService->returnNextOperGereComp($pedido);
                        
                        if (!empty($nextOper)) {
                            $this->controlePedidoColetaAceiteService->insertNextOperAceitePedCol($pedido, $nextOper[0]['CDOPERGERE']);
                            $this->enviaEmailNextOper($row, $nextOper, $pedido);
                             array_push($dataset, array(
                                "GERACOTACAO" => false,
                                "RECUSA"      => false
                            ));
                            
                            //$response->addDataSet(new DataSet('data', $dataset));
                        } else {
                            
                            $this->finalizeProcessAccepted($pedido, $response);
                            array_push($dataset, array(
                                "GERACOTACAO" => true
                               
                            ));
                            
                        }
                    } else {
                        $this->controlePedidoColetaAceiteService->updateAceitePedColRecusa($pedido, $row['OPERADORLOGADO'], $row['JUSTIFICATIVA']);
                        $this->controlePedidoColetaAceiteService->updateStatusPedCol($pedido, 'N');
                        $this->enviaEmailRecusa($row, $pedido);
                        array_push($dataset, array(
                            "GERACOTACAO" => false,
                            "RECUSA"      => true
                        ));
                        //$response->addDataSet(new DataSet('data', $dataset));
                    }
                } else {

                    if (!empty($nextOperPending)) {
                        array_push($dataset, array(
                            "GERACOTACAO" => false,
                            "RECUSA"      => false
                        ));
                        //$response->addDataSet(new DataSet('data', $dataset));
                        
                        return array("ERROR" => true, "CODE" => "OPERADOR_APROVADOR_DIF", "PARAM" => $nextOperPending[0]['NMOPERADOR']);
                    }
                }
            } else {
                return array("ERROR" => true, "CODE" => "AGUARD_APROV_FORN", "PARAM" => "");
            }
        }
        $response->addDataSet(new DataSet('data', $dataset));
    }
    
    public function finalizeProcessAccepted($row, $response){
        $this->controlePedidoColetaAceiteService->updateStatusPedCol($row, 'A');
        $dataset = array(
            "GERACOTACAO"   => true
        );
        $response->addDataSet(new DataSet('data', $dataset));
    }
    
    public function enviaEmailComprador($row){
        $pedido = $row['PEDIDOSPAI'][0];
        

        $assunto = 'Geração de Cotação  - Pedido '.$pedido["NRPEDIDOCOL"];
        $message = $this->getMessage($pedido['NRPEDIDOCOL'], $row['DATA'], $row['HORA']);
       
       
        $messageSendMail = $this->supUtilities->messageSendMail($row["FILIAL_LOGADA"], $assunto, $message, 'PORTAL E-COMMERCE');
        $address = $this->controlePedidoColetaAceiteService->returnDsEmailOper($pedido['CDOPERLANC']);
        $this->supUtilities->enviaEmail($row["EMAILLOGADO"], $assunto, $address, $messageSendMail, $address, $row["URL"], 'TermoDeAceite.pdf', 'aapplication/pdf');
    }
    
    public function enviaEmailNextOper($row, $nextOper,$pedido){
        $assunto = 'O pedido '.$pedido["NRPEDIDOCOL"] . ' está pendente de aceite.';
        $message = $this->getMessageNextOper($pedido['NRPEDIDOCOL']);
        $messageSendMail = $this->supUtilities->messageSendMail($row["FILIALLOGADA"], $assunto, $message, 'PORTAL E-COMMERCE');
        $address = $nextOper[0]['DSEMAILOPER'];
        $this->supUtilities->enviaEmail($row["EMAIL_LOGADO"], $assunto, $address, $messageSendMail, $address, $row["URL"], 'TermoDeAceite.pdf', 'aapplication/pdf');
    }
    
    public function enviaEmailRecusa($row,$pedido){
        $assunto = 'O pedido '.$pedido["NRPEDIDOCOL"] . ' foi recusado.';
        $message = $this->getMessageRecusa($pedido['NRPEDIDOCOL'], $row['NMOPERADORLOGADO'], $row['DATA'], $row['HORA']);
        $messageSendMail = $this->supUtilities->messageSendMail($row["FILIALLOGADA"], $assunto, $message, 'PORTAL E-COMMERCE');
        $address = $this->controlePedidoColetaAceiteService->returnDsEmailOper($pedido['CDOPERLANC']);
        $this->supUtilities->enviaEmail($row["EMAIL_LOGADO"], $assunto, $address, $messageSendMail, $address, $row["URL"], 'TermoDeAceite.pdf', 'aapplication/pdf');
    }
    
    private function getMessage($nrPedidoCol, $data, $hora){
        $message = "<table>
                            <tr>
                                <td> 
                                    <div>
                                        <br/> Prezado(a),
                                    </div>
                                </td>
                            </tr>
                            <tr>
                                <td class='description'>
                                    <br/>Foi gerada a cotação para o pedido <b>" . $nrPedidoCol . " após aprovação total do termo de aceite </b> em <b><span class='Object' id='OBJ_PREFIX_DWT71_com_zimbra_date'>" . $data . "</span> ". $hora ."</b>. 
                                </td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <center>
                                        <br> <p> Email automático enviado pelo Suprimentos. </p> 
                                    </center>
                                </td>
                            </tr>
                        </table>";
                        
        return $message;
    }
    
    private function getMessageNextOper($nrPedidoCol){
        $message = "<table>
                            <tr>
                                <td> 
                                    <div>
                                        <br/> Prezado(a),
                                    </div>
                                </td>
                            </tr>
                            <tr>
                                <td class='description'>
                                    <br/>O pedido <b>" . $nrPedidoCol . "</b> está pendente de aprovação. Acesse o Controle de Pedido de Coleta e realize a aprovação do termo de aceite. </b>. 
                                </td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <center>
                                        <br> <p> Email automático enviado pelo Suprimentos. </p> 
                                    </center>
                                </td>
                            </tr>
                        </table>";
                        
        return $message;
    }
    
    public function getMessageAguardandoAceite($nrPedido, $nmOperadorLogado) {
        $message      = "<table>
                            <tr>
                                <td class='title'>
                                    <div>
                                        <br/> Prezado Fornecedor,
                                    </div>
                                </td>
                            </tr>
                            <tr>
                                <td class='description'>
                                    <br/> Para prosseguirmos com o processo de cotação gentileza realizar a aprovação do termo de aceite <b>" . $nrPedido . "</b> confirmando os preços negociados.
                                </td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <center>
                                        <br>
                                        <p> Atenciosamente, </p>
                                        <p>" . $nmOperadorLogado . "</p>
                                    </center>
                                </td>
                            </tr>
                        </table>";
                    
        return $message;
    }
    
    private function getMessageRecusa($nrPedidoCol, $operadorLogado, $data, $hora){
        $message = "<table>
                            <tr>
                                <td> 
                                    <div>
                                        <br/> Prezado(a),
                                    </div>
                                </td>
                            </tr>
                            <tr>
                                <td class='description'>
                                    <br/>O termo de aceite do pedido <b>" . $nrPedidoCol . "</b> foi recusado pelo operador <b>" . $operadorLogado . "</b> em <span class='Object' id='OBJ_PREFIX_DWT71_com_zimbra_date'><b>" . $data . "</b></span> <b>". $hora ."</b>. 
                                </td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <center>
                                        <br> <p> Email automático enviado pelo Suprimentos. </p> 
                                    </center>
                                </td>
                            </tr>
                        </table>";
                        
        return $message;
    }
}